@@ -18,10 +18,15 @@ class TestModel(unittest.TestCase):
         shutil.rmtree("./results")
 
     def test_results_mean_and_prediction_interval(self):
-        model = self.deepar.train(epochs=10)
+        model = self.deepar.train(epochs=2)
         forecasts = self.deepar.predict(model)
-        mean, lower, upper = self.deepar.results(forecasts)
-        res = self.deepar.metrics(mean, lower, upper)
-        self.assertTrue(res['mean'].shape == (1, self.n, self.s))
-        self.assertTrue(res['lower'].shape == (1, self.n, self.s))
-        self.assertTrue(res['upper'].shape == (1, self.n, self.s))
\ No newline at end of file
+        samples = self.deepar.results(forecasts)
+        res = self.deepar.metrics(samples)
+
+        # Test shape of results
+        self.assertTrue(res['mase']['bottom_ind'].shape == (self.s, ))
+        self.assertTrue(res['CRPS']['bottom_ind'].shape == (self.s, ))
+        self.assertTrue(res['rmse']['bottom_ind'].shape == (self.s, ))
+
+        # Test shape of predictions
+        self.assertTrue(res['predictions']['samples']['bottom'].shape == (self.n, self.s, 500))
