@@ -4,7 +4,7 @@ from rpy2.robjects import pandas2ri
 import rpy2.robjects as ro
 from rpy2.robjects.conversion import localconverter
 import numpy as np
-from htsmodels.results.calculate_metrics import CalculateStoreResults
+from htsmodels.results.calculate_metrics import CalculateResultsMint
 import pickle
 from pathlib import Path
 from datetime import datetime
@@ -35,6 +35,7 @@ class MinT:
         dict_groups['Count'] = groups['predict']['data_matrix'].T.reshape(-1,)
         dict_groups['Date'] = np.tile(np.array(groups['dates']), (groups['train']['s'],))
         self.df = pd.DataFrame(dict_groups)
+        self.algorithm = 'mint'
 
         time_interval = (self.groups['dates'][1] - self.groups['dates'][0]).days
         if time_interval < 2:
@@ -60,11 +61,18 @@ class MinT:
         # Create directory to store results if does not exist
         Path(f'{self.input_dir}results').mkdir(parents=True, exist_ok=True)
 
-    def train(self):
+    def train(self, algorithm='ets', rec_method='mint'):
+        """Train ETS or ARIMA with conventional bottom-up or MinT reconciliation strategies
+
+        :param algorithm: ets or arima
+        :param rec_method: reconciliation method (bottom_up, mint or base)
+            base means that we just forecast all time series without any reconciliation
+        :return: df with results
+        """
         self.wall_time_preprocess = time.time() - self.timer_start
         robjects.r('''
             library('fpp3')
-            results_fn <- function(df, time, h, string_aggregate, start_predict_date) {
+            results_fn <- function(df, time, h, string_aggregate, start_predict_date, algorithm, rec_method) {
               if (time == 'quarter') {
                 fn = yearquarter
               } else if (time == 'month') {
@@ -76,6 +84,21 @@ class MinT:
               } else if (time == 'day') {
                 fn = ymd
               } 
+              
+              # Get the correct algorithm and assign the respective function
+              if (algorithm == 'ets') {
+                algo_fn = ETS
+              } else if (algorithm == 'arima') {
+                algo_fn = ARIMA
+              }
+              
+              # Get the correct reconciliation method and assign the respective function
+              if (rec_method == 'bottom_up') {
+                rec_method_fn = bottom_up
+              } else if (rec_method == 'mint') {
+                rec_method_fn = min_trace
+              }
+              
               data <- df %>%
                 mutate(Time = fn(Date)) %>%
                 select(-Date) %>%
@@ -87,27 +110,31 @@ class MinT:
               
               fit <- data_gts %>%
                 filter(Time <= fn(as.Date(start_predict_date))) %>%
-                model(base = ETS(Count)) %>%
-                reconcile(
-                  bottom_up = bottom_up(base),
-                  MinT = min_trace(base, method = "mint_shrink")
-                )
+                  model(base = algo_fn(Count)) %>%
+                  reconcile(
+                    if (rec_method == 'mint') {
+                        rec = rec_method_fn(base, method = 'mint_shrink')
+                    } else if (rec_method == 'bottom_up') {
+                        rec = rec_method_fn(base)
+                    } else {
+                    }
+                  )
               fc <- fit %>% forecast(h = h)
               
               fc_csv = fc %>% 
-                # getting the 95% interval
-                hilo(level = 95) %>%
-                unpack_hilo('95%') %>% 
-                as_tibble %>% 
-                filter(.model=='MinT') %>% 
-                select(-Count) %>% 
-                rename(lower='95%_lower') %>%
-                rename(upper='95%_upper') %>%
-                mutate(.mean=.mean) %>%
-                mutate(.mean=(sprintf("%0.2f", .mean))) %>%
-                rename(time=Time) %>%
-                lapply(as.character) %>% 
-                data.frame(stringsAsFactors=FALSE)
+                        # getting the 95% interval
+                        hilo(level = 95) %>%
+                        unpack_hilo('95%') %>% 
+                        as_tibble %>% 
+                        mutate(mean=mean(Count)) %>%
+                        mutate(std=sqrt(distributional::variance(Count))) %>%
+                        select(-Count) %>% 
+                        rename(lower='95%_lower') %>%
+                        rename(upper='95%_upper') %>%
+                        select(-.mean) %>%
+                        rename(time=Time) %>%
+                        lapply(as.character) %>% 
+                        data.frame(stringsAsFactors=FALSE)
               
               return (fc_csv)
             }
@@ -120,12 +147,24 @@ class MinT:
                                  self.time_int,
                                  self.groups['h'],
                                  self.aggregate_key,
-                                 self.last_train_date)
+                                 self.last_train_date,
+                                 algorithm,
+                                 rec_method)
         with localconverter(ro.default_converter + pandas2ri.converter):
             df_result = ro.conversion.rpy2py(df_result_r)
-        df_result[['.mean']] = df_result[['.mean']].astype('float')
+
+        # When it runs the reconciliation it also runs the base algorithm, so
+        # it has two different outputs stored in the df that we need to filter
+        if not rec_method == 'base':
+            df_result = df_result.loc[df_result['.model'] != 'base']
+
+        # name properly the models in the dataframe
+        df_result.loc[:, '.model'] = algorithm + '_' + rec_method
+
+        df_result[['mean']] = df_result[['mean']].astype('float')
         df_result[['lower']] = df_result[['lower']].astype('float')
         df_result[['upper']] = df_result[['upper']].astype('float')
+        df_result[['std']] = df_result[['std']].astype('float')
         self.wall_time_build_model = time.time() - self.timer_start - self.wall_time_preprocess
         self.wall_time_train = time.time() - self.timer_start - self.wall_time_build_model
         self.wall_time_predict = time.time() - self.timer_start - self.wall_time_train
@@ -158,38 +197,23 @@ class MinT:
         cols.append('Date')
         # Zip can sometimes have the dtype int and breaks
         pred_mint = pred_mint.astype({k: 'string' for k in cols})
-        self.df = self.df.astype({k: 'string' for k in cols})
         pred_mint['Date'] = pd.to_datetime(pred_mint['Date'])
-        self.df['Date'] = pd.to_datetime(self.df['Date'])
-
-        res_joined = self.df.merge(pred_mint, how='left', on=cols)
 
         # Filter only the predictions
-        res_joined = res_joined[res_joined['Date'] > self.last_train_date]
-        pred = res_joined['.mean'].to_numpy().reshape(self.groups['train']['s'], self.groups['h']).T
-        lower = res_joined['lower'].to_numpy().reshape(self.groups['train']['s'], self.groups['h']).T
-        upper = res_joined['lower'].to_numpy().reshape(self.groups['train']['s'], self.groups['h']).T
-        pred_complete = np.concatenate((np.zeros((self.groups['train']['n'],
-                                                  self.groups['train']['s'])), pred), axis=0)[np.newaxis, :, :]
-        lower_complete = np.concatenate((np.zeros((self.groups['train']['n'],
-                                                  self.groups['train']['s'])), lower), axis=0)[np.newaxis, :, :]
-        upper_complete = np.concatenate((np.zeros((self.groups['train']['n'],
-                                                  self.groups['train']['s'])), upper), axis=0)[np.newaxis, :, :]
+        pred_mint = pred_mint[pred_mint['Date'] > self.last_train_date]
 
-        return pred_complete, lower_complete, upper_complete
+        return pred_mint
 
     def store_metrics(self, res):
         with open(f'{self.input_dir}results/results_gp_cov_{self.dataset}.pickle', 'wb') as handle:
             pickle.dump(res, handle, pickle.HIGHEST_PROTOCOL)
 
-    def metrics(self, mean, lower, upper):
-        calc_results = CalculateStoreResults(mean, self.groups)
+    def metrics(self, df_results_mint):
+        calc_results = CalculateResultsMint(df_results_mint=df_results_mint,
+                                            groups=self.groups)
         res = calc_results.calculate_metrics()
         self.wall_time_total = time.time() - self.timer_start
 
-        res['mean'] = mean
-        res['lower'] = lower
-        res['upper'] = upper
         res['wall_time'] = {}
         res['wall_time']['wall_time_preprocess'] = self.wall_time_preprocess
         res['wall_time']['wall_time_build_model'] = self.wall_time_build_model
